import cv2
import mediapipe as mp
import time
import math

class PoseDetector():

    def __init__(self,mode=False, upBody = False, smooth=True,
                 detectionCon=0.5, trackCon=0.5):
        self.mode = mode
        self.upperBody = upBody
        self.smooth = smooth
        self.detectionCon = detectionCon
        self.trackCon = trackCon

        self.mpDraw = mp.solutions.drawing_utils
        self.mpPose = mp.solutions.pose
        #self.pose = self.mpPose.Pose(self.mode,self.upperBody,self.smooth,self.detectionCon,self.trackCon)
        self.pose = self.mpPose.Pose()

    def findPose(self,img,draw=True):
        imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        self.results = self.pose.process(imgRGB)
        if self.results.pose_landmarks:
            if draw:
                self.mpDraw.draw_landmarks(img,self.results.pose_landmarks,self.mpPose.POSE_CONNECTIONS)

        return img
    def findPosition(self,img,draw=True):
        self.lmlist = []
        if self.results.pose_landmarks:
            for id,lm in enumerate(self.results.pose_landmarks.landmark):
                h,w,c = img.shape
                #print(id, lm)
                cx,cy = int(lm.x*w), int(lm.y*h)
                self.lmlist.append([id,cx,cy])
                if draw:
                    cv2.circle(img,(cx,cy),5,(255,0,0),cv2.FILLED)
        return self.lmlist
    def findAngle(self,img,p1,p2,p3,draw=True):
        #Get the Landmarks
        x1,y1 = self.lmlist[p1][1:]
        x2,y2 = self.lmlist[p2][1:]
        x3, y3 = self.lmlist[p3][1:]
        # calculate Angle
        angle = math.degrees(math.atan2(y3-y2,x3-x2) - math.atan2(y1-y2,x1-x2))

        if angle < 0:
            angle += 360
        #print(angle)
        # Draw
        if draw:
            cv2.line(img,(x1, y1),(x2, y2),(255,0,0),3)
            cv2.line(img, (x2, y2), (x3, y3), (255, 0, 0), 3)
            cv2.circle(img, (x1, y1), 10, (255, 0, 0), cv2.FILLED)
            cv2.circle(img, (x1, y1), 15, (255, 0, 0), 2)
            cv2.circle(img, (x2, y2), 10, (255, 0, 0), cv2.FILLED)
            cv2.circle(img, (x2, y2), 15, (255, 0, 0), 2)
            cv2.circle(img, (x3, y3), 10, (255, 0, 0), cv2.FILLED)
            cv2.circle(img, (x3, y3), 15, (255, 0, 0), 2)
            cv2.putText(img,str(round(angle,2)),(x2-50,y2+50),cv2.FONT_HERSHEY_PLAIN,2,(0,0,255),2)
        return angle

def main():
    cap = cv2.VideoCapture(1)
    pTime = 0
    detector = PoseDetector()
    while True:
        ret, img = cap.read()
        img = detector.findPose(img)
        lmlist = detector.findPosition(img,draw=True)
        if lmlist:
            print(lmlist[14])
            cx,cy = lmlist[14][1],lmlist[14][2]
            cv2.circle(img, (cx, cy), 10, (255, 0, 255), cv2.FILLED)
            cv2.putText(img, 'cx=' + str(cx) + " cy=" + str(cy), (800, 50), cv2.FONT_HERSHEY_PLAIN, 3, (0, 0, 255), 3)
        cTime = time.time()
        fps = int(1 / (cTime - pTime))
        pTime = cTime
        print(fps)
        cv2.putText(img, "Fps = " + str(fps), (70, 50), cv2.FONT_HERSHEY_PLAIN, 3, (255, 0, 0), 3)

        cv2.imshow('Frame', img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break
    cap.release()
    cv2.destroyAllWindows()
if __name__ == "__main__":
    main()



import cv2
import mediapipe as mp
import time

class FaceMeshDetector():
    def __init__(self, staticmode=False,maxFaces=1,minDetectionCon=0.5,minTrackCon=0.5 ):
        self.staticmode=staticmode
        self.maxFaces=maxFaces
        self.minDetectionCon=minDetectionCon
        self.minTrackCon=minTrackCon
        self.mpDraw = mp.solutions.drawing_utils
        self.mpFaceMesh = mp.solutions.face_mesh
        self.faceMesh = self.mpFaceMesh.FaceMesh(self.staticmode,self.maxFaces,
                                                 self.minDetectionCon,self.minTrackCon)
        self.drawSpec = self.mpDraw.DrawingSpec(color=(0, 255, 0),
                                                thickness=2,circle_radius=2)

    def findFaceMesh(self,img,draw=True):
        imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        self.results= self.faceMesh.process(imgRGB)
        faces = []
        if self.results.multi_face_landmarks:
            for faceLms in self.results.multi_face_landmarks:
                print(faceLms)
                if draw:
                    self.mpDraw.draw_landmarks(img,faceLms,self.mpFaceMesh.FACEMESH_CONTOURS,
                                               self.drawSpec,self.drawSpec)
                face = []
                for id,lm in enumerate(faceLms.landmark):
                    #print(lm.x,lm.y)
                    ih,iw,ic = img.shape
                    x,y = int(lm.x*iw),int(lm.y*ih)
                    #print(id,x,y)
                    face.append((x,y))
                faces.append(face)
        return img,faces

def main():
    cap = cv2.VideoCapture(1)
    cap.set(3, 800)
    cap.set(4, 600)
    pTime = 0
    detector = FaceMeshDetector()
    while cap.isOpened():
        ret, img = cap.read()
        img,faces = detector.findFaceMesh(img)
        if len(faces)!=0:
            print(len(faces))
        cTime = time.time()
        fps = 1 / (cTime - pTime)
        pTime = cTime
        cv2.putText(img, f'FPS:{int(fps)}', (20, 40), cv2.FONT_HERSHEY_PLAIN,
                    3, (0, 255, 0), 2)
        cv2.imshow("Image", img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break
    cap.release()
    cv2.destroyAllWindows()

if __name__ == "__main__":
    main()


"""
Face Detection Module
By: GB Softronics Solution
Website: https://www.gbsoftronics.com/
"""

import cv2
import mediapipe as mp
import time

class FaceDetector():
    """
        Finds Faces using the mediapipe library. Exports the landmarks
        in faces. Adds extra functionalities as draw face using fancyDraw Function.
        """
    def __init__(self,minDetectionCon=0.5):
        """
        :param minDetectionCon: MinimumDetection Confidance. Range between 0-1
        """
        self.minDetectionCon = minDetectionCon
        self.mpFaceDetection = mp.solutions.face_detection
        self.mpDraw = mp.solutions.drawing_utils
        self.faceDetection = self.mpFaceDetection.FaceDetection(self.minDetectionCon)

    def findFaces(self,img,draw=True):
        """
        :param img: input Image in BGR Format
        :param draw: Flag to draw the face bounding box. Default: True
        """
        imgRGB = cv2.cvtColor(img,cv2.COLOR_BGR2RGB)
        self.results= self.faceDetection.process(imgRGB)
        #print(results)
        bboxs = []
        if self.results.detections:
            for id,detection in enumerate(self.results.detections):
                #mpDraw.draw_detection(img,detection)
                #print(id,detection)
                bboxC = detection.location_data.relative_bounding_box
                ih,iw,ic = img.shape
                bbox = int(bboxC.xmin * iw), int(bboxC.ymin * ih), int(bboxC.width * iw), int(bboxC.height * ih)
                bboxs.append([bbox,detection.score])
                #print(bbox)
                if draw:
                    #cv2.rectangle(img,bbox,(255,0,255),5)
                    self.fancyDraw(img,bbox)
                    confidance_score = int(detection.score[0] * 100)
                    #print(confidance_score)
                    cv2.putText(img, str(confidance_score) + '%', (bbox[0],bbox[1]-20), cv2.FONT_HERSHEY_PLAIN, 4,
                                (0, 0, 255), 2)
        return img, bboxs

    def fancyDraw(self, img, bbox, l=50, t=10, rt=1):
        """
        :param img: input Image in BGR Format
        :param bbox: bounding box as tuple (x,y,w,h)
        :param l: Length of Line for fancy Rect Draw
        :param t: Thickness of Line for Fancy Rect Draw. Max value, thicker line
        :param rt: Rectangle Thickness
        """
        x, y, w, h = bbox
        x1, y1 = x + w, y + h

        cv2.rectangle(img, bbox, (255, 0, 255), rt)
        # Top Left x,y
        cv2.line(img, (x, y), (x + l, y), (255, 0, 255), t)
        cv2.line(img, (x, y), (x, y + l), (255, 0, 255), t)
        # Top Right x1,y
        cv2.line(img, (x1, y), (x1 - l, y), (255, 0, 255), t)
        cv2.line(img, (x1, y), (x1, y + l), (255, 0, 255), t)
        # Bottom Left x,y1
        cv2.line(img, (x, y1), (x + l, y1), (255, 0, 255), t)
        cv2.line(img, (x, y1), (x, y1 - l), (255, 0, 255), t)
        # Bottom Right x1,y1
        cv2.line(img, (x1, y1), (x1 - l, y1), (255, 0, 255), t)
        cv2.line(img, (x1, y1), (x1, y1 - l), (255, 0, 255), t)
        return img

def main():
    cap = cv2.VideoCapture(0)
    pTime = 0
    detector = FaceDetector()
    while cap.isOpened():
        ret, img = cap.read()
        if not ret:
            continue

        img,bboxs = detector.findFaces(img)

        cTime = time.time()
        fps = 1 / (cTime - pTime)
        pTime = cTime
        cv2.putText(img, f'FPS: {int(fps)}', (20, 30), cv2.FONT_HERSHEY_PLAIN, 2,
                    (0, 0, 255), 4)
        cv2.imshow("Image", img)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()
if __name__ == "__main__":
    main()